package marvel.centralsmashdown.game.control
{
   import com.gs.TweenLite;
   import com.markdavies.debug.DebugBox;
   import com.markdavies.events.CustomEvent;
   import fl.motion.easing.*;
   import flash.display.*;
   import flash.errors.*;
   import flash.events.*;
   import flash.utils.*;
   import marvel.centralsmashdown.*;
   import marvel.centralsmashdown.game.*;
   import marvel.centralsmashdown.game.sprites.*;
   import marvel.centralsmashdown.game.views.*;
   import marvel.centralsmashdown.views.*;
   
   public class GameControl
   {
      
      public static var COMICTEXT_TYPE_CRASH:String = "crash";
      
      public static var RUMBLE_PERC_MAX_DROPHEIGHT:int = 2000;
      
      public static var ENEMY_MAX_Y:int = -200;
      
      public static var COMICTEXT_TYPE_DOUBLEUP:String = "doubleup";
      
      public static var MAX_ENEMIES:int = 10;
      
      public static var BONUS_TYPE_TIMEUP:String = "timeup";
      
      public static var ENEMY_TYPE_SNAKE:String = "snake";
      
      public static var ENEMY_TYPE_MOVER:String = "mover";
      
      public static var GAME_MODE_CHALLENGE:String = "gameModeChallenge";
      
      private static var instance:GameControl = new GameControl();
      
      public static var ENEMY_REMOVE_RANGE:int = 1000;
      
      public static var GAME_WIDTH:int = 900;
      
      public static var GROUND_Y:int = -35;
      
      public static var PLAYER_ENEMY_HIT_SIDEWAYS_RATE:Number = 0.25;
      
      public static var PLAYER_ENEMY_HIT_POWER:Number = 0.75;
      
      public static var RUMBLE_BROWSER_SHAKE_PERC:Number = 0.9;
      
      public static var COMICTEXT_TYPE_SMASH:String = "smash";
      
      public static var RUMBLE_MAX_AMOUNT:int = 40;
      
      public static var SCORE_PER_ENEMY:Number = 150;
      
      public static var ENEMY_CREATION_CHANCE:Number = 0.05;
      
      public static var HIT_DISTANCE:int = 70;
      
      public static var COMBO_HIT_TIME:int = 500;
      
      public static var MIN_COMBO_FOR_BONUS:Number = 2;
      
      public static var BONUS_TYPE_DOUBLEUP:String = "doubleup";
      
      public static var TIMEUP_AMOUNT:Number = 10000;
      
      public static var ENEMY_TYPE_FLOATER:String = "floater";
      
      public static var GAME_MODE_ARCADE:String = "gameModeArcade";
      
      public static var COMICTEXT_TYPE_COMBO:String = "combo";
      
      public static var COMICTEXT_TYPE_BOOM:String = "boom";
      
      public static var RUMBLE_MIN_AMOUNT:int = 10;
      
      public static var ENEMY_CREATION_RANGE:int = 300;
      
      public static var ENEMY_START_PADDING_X:int = 100;
      
      public static var ENEMY_START_PADDING_Y:int = 200;
      
      public static var ENEMY_TYPE_FOLLOWER:String = "follower";
      
      public static var COMICTEXT_TYPE_TIMEUP:String = "timeup";
      
      public static var RUMBLE_OUTERFRAME_SHAKE_PERC:Number = 0.5;
       
      
      private var _player:Player;
      
      private var _highScoreArcade:Number = 0;
      
      private var _pauseStarted:Date;
      
      private var _contentHolder:MovieClip;
      
      private var _bestChallengeScore:int;
      
      private var _score:int;
      
      private var _gameMode:String;
      
      private var _main:Main;
      
      private var _gameScreen:GameScreen;
      
      private var _bestArcadeScore:int;
      
      private var _currentTime:int;
      
      private var _quitToMain:MovieClip;
      
      private var _timeRemaining:int;
      
      private var _timeStarted:Date;
      
      private var _gameOverTimer:Timer;
      
      private var _comboCount:int;
      
      private var _pausedTime:int;
      
      private var _pauseScreen:MovieClip;
      
      private var _miniMap:MiniMap;
      
      private var _outerFrame:OuterFrame;
      
      private var _gameCamera:GameCamera;
      
      private var _isPaused:Boolean = false;
      
      private var _highScoreChallenge:Number = 0;
      
      private var _rumblePack:RumblePack;
      
      private var _lastEnemyType:String;
      
      private var _bottomNav:BottomNav;
      
      private var _comboTime:Date;
      
      private var _stage:Stage;
      
      public function GameControl()
      {
         _isPaused = false;
         _highScoreArcade = 0;
         _highScoreChallenge = 0;
         super();
         if(instance)
         {
            throw new Error("GameControl can only be accessed through GameControl.getInstance()");
         }
         _main = Main.getInstance();
      }
      
      public static function get lastEnemyType() : String
      {
         return instance._lastEnemyType;
      }
      
      public static function getInstance() : GameControl
      {
         return instance;
      }
      
      private function spriteGroundHitHandler(param1:CustomEvent) : void
      {
         var _loc2_:Number = NaN;
         if(param1.data.target is Player)
         {
            _loc2_ = -_player.dropHeight / GameControl.RUMBLE_PERC_MAX_DROPHEIGHT;
            if(_loc2_ > 1)
            {
               _loc2_ = 1;
            }
            rumble(_loc2_);
            _main.sc.play("s_hitground",_loc2_);
            if(_gameMode == GameControl.GAME_MODE_CHALLENGE && _player.enemiesHitThisJump > 0)
            {
               if(_score > _bestChallengeScore)
               {
                  _bestChallengeScore = _score;
               }
               gameOver();
            }
            _player.setToMaxY();
            _player.power = 0;
            _player.isDoubleUp = false;
         }
         else if(param1.data.target is Enemy)
         {
            Enemy(param1.data.target).setToMaxY();
         }
      }
      
      private function enemyDieHandler(param1:Event) : void
      {
         var _loc2_:Enemy = null;
         if(param1.target is Enemy)
         {
            _loc2_ = Enemy(param1.target);
            _loc2_.removeEventListener(GameSprite.EVENT_GROUND_HIT,spriteGroundHitHandler);
            _loc2_.removeEventListener(GameSprite.EVENT_OUTSIDE_BORDERS,spriteOutsideBorderHandler);
            _loc2_.removeEventListener(Event.REMOVED,enemyDieHandler);
            _miniMap.remove(_loc2_);
         }
      }
      
      public function get highScore() : int
      {
         var _loc1_:int = 0;
         switch(gameMode)
         {
            case GameControl.GAME_MODE_ARCADE:
               _loc1_ = highScoreArcade;
               break;
            case GameControl.GAME_MODE_CHALLENGE:
               _loc1_ = highScoreChallenge;
         }
         return _loc1_;
      }
      
      private function rumble(param1:Number) : void
      {
         var _loc2_:Number = NaN;
         var _loc3_:Array = null;
         var _loc4_:Number = NaN;
         var _loc5_:Number = NaN;
         var _loc6_:Number = NaN;
         var _loc7_:int = 0;
         var _loc8_:String = null;
         var _loc9_:Number = NaN;
         _loc2_ = (GameControl.RUMBLE_MAX_AMOUNT - GameControl.RUMBLE_MIN_AMOUNT) * param1 + GameControl.RUMBLE_MIN_AMOUNT;
         if(param1 > GameControl.RUMBLE_OUTERFRAME_SHAKE_PERC)
         {
            _loc3_ = [MovieClip(_main.border),MovieClip(_main.outerFrame),MovieClip(_gameScreen)];
            if((_loc9_ = Math.random() * 3) < 1)
            {
               _loc8_ = GameControl.COMICTEXT_TYPE_BOOM;
            }
            else if(_loc9_ < 2)
            {
               _loc8_ = GameControl.COMICTEXT_TYPE_CRASH;
            }
            else
            {
               _loc8_ = GameControl.COMICTEXT_TYPE_SMASH;
            }
            makeComicText(_player.x - 20,_player.y - 120,_loc8_);
            makeCrack();
         }
         else
         {
            _loc3_ = [MovieClip(_gameScreen)];
         }
         _loc7_ = 0;
         while(_loc7_ < _loc3_.length)
         {
            _rumblePack = new RumblePack();
            if(_loc3_[_loc7_] == _gameScreen)
            {
               _loc4_ = _gameScreen.y;
               _loc5_ = GameCamera.CAMERA_HEIGHT + GameCamera.CAMERA_Y;
               _loc6_ = GameControl.GAME_WIDTH;
            }
            else
            {
               _loc4_ = 0;
               _loc5_ = Number(_loc3_[_loc7_].root.stage.stageHeight);
               _loc6_ = Number(_loc3_[_loc7_].root.stage.stageWidth);
            }
            _rumblePack.create(_loc3_[_loc7_],_loc6_,_loc5_,0,_loc4_);
            _rumblePack.startRumble(_loc2_,1);
            if(_loc3_[_loc7_] == _gameScreen)
            {
               _rumblePack.y = -_loc4_;
            }
            _loc3_[_loc7_].addChildAt(_rumblePack,_loc3_[_loc7_].numChildren);
            _loc7_++;
         }
      }
      
      private function pauseToggle() : void
      {
         var _loc1_:Date = null;
         _main.clearContent();
         if(_isPaused == false)
         {
            _pauseScreen = new PauseScreen();
            _pauseScreen.btnPaused.addEventListener(MouseEvent.MOUSE_DOWN,pauseClickHandler);
            _pauseScreen.btnPaused.buttonMode = true;
            _contentHolder.addChild(_pauseScreen);
            _pauseStarted = new Date();
            _main.sc.setVolume("s_soundtrack_game",0);
         }
         else
         {
            _loc1_ = new Date();
            _pausedTime += _loc1_.getTime() - _pauseStarted.getTime();
            _main.sc.setVolume("s_soundtrack_game",1);
         }
         _isPaused = !_isPaused;
      }
      
      public function get score() : int
      {
         return _score;
      }
      
      public function get lastScore() : int
      {
         return _score;
      }
      
      public function exit() : void
      {
         _main.gameOverHandler();
      }
      
      private function makeComicText(param1:Number, param2:Number, param3:String, param4:Number = 0) : void
      {
         var _loc5_:ComicTextDisplay = null;
         (_loc5_ = new ComicTextDisplay()).type = param3;
         _loc5_.x = param1;
         _loc5_.y = param2;
         _loc5_.multiplier = param4;
         _loc5_.show();
         _gameScreen.addChild(_loc5_);
      }
      
      private function render() : void
      {
         var _loc1_:Number = NaN;
         var _loc2_:GameSprite = null;
         _outerFrame.score = _score;
         if(_gameMode == GameControl.GAME_MODE_ARCADE)
         {
            _outerFrame.timeRemaining = _timeRemaining;
            _outerFrame.timePercentage = _timeRemaining / (Config.TIMER_MODE_TIMEPERGAME * 1000);
         }
         _loc1_ = 0;
         while(_loc1_ < _gameScreen.spriteHolder.numChildren)
         {
            _loc2_ = GameSprite(_gameScreen.spriteHolder.getChildAt(_loc1_));
            _loc2_.move();
            if(_loc2_.currentStatus != GameSprite.STATUS_DEAD)
            {
               _miniMap.set(_loc2_);
            }
            _loc1_++;
         }
         _gameScreen.x = _gameCamera.x;
         _gameScreen.y = _gameCamera.y;
         _gameScreen.scaleX = _gameScreen.scaleY = _gameCamera.scale;
         _gameScreen.render();
      }
      
      private function bonusHitHandler(param1:CustomEvent) : void
      {
         var _loc2_:BonusSprite = null;
         var _loc3_:Number = NaN;
         _loc2_ = BonusSprite(param1.data.bonus);
         _loc2_.currentStatus = GameSprite.STATUS_DEAD;
         _loc2_.gotoAndPlay("die");
         _miniMap.remove(_loc2_);
         _loc3_ = _player.speedY * 10;
         if(_loc2_.type == GameControl.BONUS_TYPE_DOUBLEUP)
         {
            makeComicText(_player.x - 20,_player.y + _loc3_,GameControl.COMICTEXT_TYPE_DOUBLEUP);
            _player.isDoubleUp = true;
         }
         else if(_loc2_.type == GameControl.BONUS_TYPE_TIMEUP)
         {
            _timeStarted.setTime(_timeStarted.getTime() + GameControl.TIMEUP_AMOUNT);
            makeComicText(_player.x - 20,_player.y + _loc3_,GameControl.COMICTEXT_TYPE_TIMEUP);
         }
         TweenLite.to(_loc2_,0.5,{
            "scaleX":2,
            "scaleY":2,
            "alpha":0,
            "ease":Bounce.easeOut,
            "onComplete":_loc2_.die
         });
      }
      
      private function pauseClickHandler(param1:MouseEvent) : void
      {
         pauseToggle();
      }
      
      public function get gameMode() : String
      {
         return _gameMode;
      }
      
      public function get paused() : Boolean
      {
         return _isPaused;
      }
      
      public function engine(param1:Event) : void
      {
         if(_isPaused == true)
         {
            return;
         }
         calculate();
         render();
      }
      
      private function spriteOutsideBorderHandler(param1:CustomEvent) : void
      {
         var _loc2_:Enemy = null;
         if(param1.data.target is Enemy)
         {
            _loc2_ = Enemy(param1.data.target);
            switch(_loc2_.enemyType)
            {
               case GameControl.ENEMY_TYPE_FOLLOWER:
                  _loc2_.currentStatus = GameSprite.STATUS_DEAD;
                  _loc2_.gotoAndPlay("die");
                  break;
               case GameControl.ENEMY_TYPE_FLOATER:
                  _loc2_.x = param1.data.direction == 0 ? _loc2_.minXPos + GameControl.ENEMY_START_PADDING_X : _loc2_.maxXpos - GameControl.ENEMY_START_PADDING_X;
                  _loc2_.aimSpeedX -= _loc2_.aimSpeedX;
            }
         }
         else if(param1.data.target is Player)
         {
            if(_player.x > _player.maxXPos)
            {
               _player.x = _player.maxXPos;
            }
            if(_player.x < _player.minXPos)
            {
               _player.x = _player.minXPos;
            }
         }
      }
      
      public function get highScoreArcade() : int
      {
         return _highScoreArcade;
      }
      
      private function gameOver() : void
      {
         var _loc1_:GameOver = null;
         if(_gameMode == GameControl.GAME_MODE_ARCADE)
         {
            _gameOverTimer = new Timer(3000,1);
            _gameOverTimer.addEventListener(TimerEvent.TIMER,out);
            _gameOverTimer.start();
         }
         _loc1_ = new GameOver();
         _loc1_.mode = _gameMode;
         TweenLite.from(_loc1_,0.5,{
            "alpha":0,
            "delay":1.25
         });
         _contentHolder.addChild(_loc1_);
         _main.bottomNav.menu = Main.MENU_MODE_SCREEN;
         end();
      }
      
      public function get highScoreChallenge() : int
      {
         return _highScoreChallenge;
      }
      
      private function quitToMainClickHandler(param1:CustomEvent) : void
      {
         switch(param1.data)
         {
            case Config.NAV_QUITMAIN_YES:
               _bottomNav.dispatchEvent(new CustomEvent(Main.EVENT_NAV_CLICK,Config.NAV_QUITMAIN_YES));
               end();
               break;
            case Config.NAV_QUITMAIN_NO:
               _contentHolder.removeChild(_quitToMain);
               _isPaused = false;
         }
      }
      
      private function reset() : void
      {
         _score = 0;
         _bestArcadeScore = 0;
         _player = new Player();
         _player.reset();
         _player.addEventListener(Event.REMOVED_FROM_STAGE,remHandler);
         _gameScreen.spriteHolder.addChild(_player);
      }
      
      private function remHandler(param1:Event) : void
      {
         trace("removed!");
      }
      
      private function out(param1:TimerEvent) : void
      {
         _gameOverTimer.removeEventListener(TimerEvent.TIMER,out);
         exit();
      }
      
      public function end() : void
      {
         _miniMap.clearAll();
         _miniMap.showing = false;
         while(_gameScreen.crackHolder.numChildren > 0)
         {
            _gameScreen.crackHolder.removeChildAt(0);
         }
         switch(gameMode)
         {
            case GameControl.GAME_MODE_ARCADE:
               if(_score > highScoreArcade)
               {
                  highScoreArcade = _score;
               }
               break;
            case GameControl.GAME_MODE_CHALLENGE:
               if(_score > highScoreChallenge)
               {
                  highScoreChallenge = _score;
               }
         }
         registerListeners(false);
         _main.so = {
            "highScoreArcade":_highScoreArcade,
            "highScoreChallenge":_highScoreChallenge
         };
         _stage.quality = StageQuality.HIGH;
      }
      
      private function registerListeners(param1:Boolean) : void
      {
         var _loc2_:Function = null;
         _loc2_ = param1 == true ? _main.addEventListener : _main.removeEventListener;
         _loc2_(Event.ENTER_FRAME,engine);
         _loc2_ = param1 == true ? _gameScreen.stage.addEventListener : _gameScreen.stage.removeEventListener;
         _loc2_(MouseEvent.MOUSE_DOWN,_player.mousePressHandler);
         _loc2_(MouseEvent.MOUSE_UP,_player.mouseReleaseHandler);
         _loc2_ = param1 == true ? _player.addEventListener : _player.removeEventListener;
         _loc2_(GameSprite.EVENT_GROUND_HIT,spriteGroundHitHandler);
         _loc2_(GameSprite.EVENT_ENEMY_HIT,enemyHitHandler);
         _loc2_(GameSprite.EVENT_BONUS_HIT,bonusHitHandler);
         _loc2_(GameSprite.EVENT_IS_HITTING_SPRITE,spritesHittingHandler);
         _loc2_(GameSprite.EVENT_OUTSIDE_BORDERS,spriteOutsideBorderHandler);
         _loc2_ = param1 == true ? _stage.addEventListener : _stage.removeEventListener;
         _loc2_(KeyboardEvent.KEY_DOWN,keyDownHandler);
         _loc2_ = param1 == true ? _bottomNav.addEventListener : _bottomNav.removeEventListener;
         _loc2_(Main.EVENT_NAV_CLICK,navClickHandler);
      }
      
      private function quitToMainScreen() : void
      {
         _main.clearContent();
         _quitToMain = new QuitToMain();
         _quitToMain.addEventListener(Main.EVENT_NAV_CLICK,quitToMainClickHandler,false,0,true);
         _contentHolder.addChild(_quitToMain);
         _isPaused = true;
      }
      
      private function enemyHitHandler(param1:CustomEvent) : void
      {
         var _loc2_:Enemy = null;
         var _loc3_:Object = null;
         var _loc4_:int = 0;
         var _loc5_:int = 0;
         _loc2_ = Enemy(param1.data.enemy);
         _loc3_ = _player.calculateRelativePosition(_loc2_);
         _loc2_.currentStatus = GameSprite.STATUS_DEAD;
         _loc2_.gotoAndPlay("die");
         _miniMap.remove(_loc2_);
         if(_gameMode == GameControl.GAME_MODE_ARCADE)
         {
            _loc4_ = _player.isDoubleUp == true ? int(GameControl.SCORE_PER_ENEMY * 2) : int(GameControl.SCORE_PER_ENEMY);
         }
         else
         {
            _loc4_ = _player.isDoubleUp == true ? 2 : 1;
         }
         _score += _loc4_;
         _player.jump(GameControl.PLAYER_ENEMY_HIT_POWER);
         _player.accelerationX = -_loc3_.adj * GameControl.PLAYER_ENEMY_HIT_SIDEWAYS_RATE;
         _comboTime = new Date();
         _comboCount += _player.isDoubleUp == true ? 2 : 1;
         _loc5_ = Math.round(Math.random() * 3) + 1;
         _main.sc.play("s_explosion" + _loc5_.toString());
         if(Math.random() > 0.75)
         {
            _loc5_ = Math.round(Math.random() * 3) + 1;
            _main.sc.play("s_damage" + _loc5_.toString());
         }
         DebugBox.debug("combo",_comboCount);
      }
      
      private function createSprite(param1:Class) : void
      {
         var _loc2_:Enemy = null;
         var _loc3_:BonusSprite = null;
         switch(param1)
         {
            case Enemy:
               _loc2_ = new Enemy();
               _loc2_.chooseType();
               _loc2_.init(_player);
               _lastEnemyType = _loc2_.enemyType;
               _loc2_.addEventListener(GameSprite.EVENT_GROUND_HIT,spriteGroundHitHandler);
               _loc2_.addEventListener(GameSprite.EVENT_OUTSIDE_BORDERS,spriteOutsideBorderHandler);
               _loc2_.addEventListener(Event.REMOVED,enemyDieHandler);
               _gameScreen.spriteHolder.addChild(_loc2_);
               break;
            case BonusSprite:
               _loc3_ = new BonusSprite();
               _loc3_.x = _player.x;
               _loc3_.y = _player.y;
               _loc3_.type = _gameMode == GameControl.GAME_MODE_CHALLENGE || Math.random() > 0.5 ? GameControl.BONUS_TYPE_DOUBLEUP : GameControl.BONUS_TYPE_TIMEUP;
               _loc3_.currentStatus = GameSprite.STATUS_CREATING;
               TweenLite.from(_loc3_,0.5,{
                  "scaleX":0,
                  "scaleY":0,
                  "delay":0.5,
                  "ease":Bounce.easeOut,
                  "onComplete":_loc3_.ready
               });
               _gameScreen.spriteHolder.addChild(_loc3_);
         }
      }
      
      public function start() : void
      {
         var _loc1_:int = 0;
         if(_gameScreen == null)
         {
            throw new Error("GameControl can only run if there is a registered GameScreen!");
         }
         _outerFrame.gameMode = gameMode;
         _gameCamera = GameCamera.getInstance();
         _gameCamera.reset();
         _comboTime = new Date();
         _timeStarted = new Date();
         _pausedTime = 0;
         _comboCount = 0;
         _score = 0;
         _bestArcadeScore = 0;
         _outerFrame.scoreHardChange = 0;
         _miniMap.showing = true;
         _miniMap.clearAll();
         _isPaused = false;
         registerListeners(true);
         _stage.quality = StageQuality.MEDIUM;
         _loc1_ = Math.round(Math.random()) + 1;
         _main.sc.play("s_hulksmash" + _loc1_.toString());
         _main.bottomNav.menu = Main.MENU_MODE_GAME;
         DebugBox.setAtTop();
      }
      
      public function registerScreens() : void
      {
         _gameScreen = _main.gameScreen;
         _outerFrame = _main.outerFrame;
         _miniMap = _outerFrame.miniMap;
         _bottomNav = _outerFrame.bottomNav;
         _contentHolder = _main.contentHolder;
         _stage = _main.stage;
         reset();
      }
      
      private function navClickHandler(param1:CustomEvent) : void
      {
         switch(param1.data)
         {
            case Config.NAV_PAUSEGAME:
               pauseToggle();
               break;
            case Config.NAV_MAINMENU:
               quitToMainScreen();
         }
      }
      
      public function set highScoreChallenge(param1:int) : void
      {
         _highScoreChallenge = param1;
      }
      
      private function comboHandler(param1:Number) : void
      {
         if(_gameMode == GameControl.GAME_MODE_ARCADE)
         {
            _score += (param1 - 1) * GameControl.SCORE_PER_ENEMY;
         }
         else
         {
            _score += param1 - 1;
         }
         makeComicText(_player.x,_player.y - 50,GameControl.COMICTEXT_TYPE_COMBO,param1);
      }
      
      private function makeCrack() : *
      {
         var _loc1_:Crack = null;
         _loc1_ = new Crack();
         _loc1_.y = GameControl.GROUND_Y + 37;
         _loc1_.x = _player.x;
         _loc1_.gotoAndStop(Math.round(Math.random() * 3) + 1);
         _gameScreen.crackHolder.addChild(_loc1_);
      }
      
      private function keyDownHandler(param1:KeyboardEvent) : void
      {
         var _loc2_:int = 0;
         _loc2_ = 5;
         switch(param1.keyCode)
         {
            case 80:
               pauseToggle();
               break;
            case 37:
               if(_main.debugMode == true)
               {
                  _gameCamera.xOffset -= _loc2_;
               }
               break;
            case 39:
               if(_main.debugMode == true)
               {
                  _gameCamera.xOffset += _loc2_;
               }
               break;
            case 38:
               if(_main.debugMode == true)
               {
                  _gameCamera.yOffset -= _loc2_;
               }
               break;
            case 40:
               if(_main.debugMode == true)
               {
                  _gameCamera.yOffset += _loc2_;
               }
         }
      }
      
      public function set gameMode(param1:String) : void
      {
         _gameMode = param1;
      }
      
      public function set highScoreArcade(param1:int) : void
      {
         _highScoreArcade = param1;
      }
      
      private function calculate() : void
      {
         var _loc1_:Date = null;
         var _loc2_:int = 0;
         var _loc3_:int = 0;
         var _loc4_:Boolean = false;
         var _loc5_:Number = NaN;
         var _loc6_:int = 0;
         var _loc7_:GameSprite = null;
         var _loc8_:Date = null;
         _player.checkPower();
         _player.checkDoubleUp();
         _player.checkOtherSprites();
         _gameCamera.calculate(_player.x,_player.y,_player.speedX,_player.speedY);
         _loc1_ = new Date();
         if(_loc1_.getTime() - _comboTime.getTime() > GameControl.COMBO_HIT_TIME)
         {
            if(_comboCount > 1)
            {
               comboHandler(_comboCount);
               _loc6_ = _player.isDoubleUp == true ? int(GameControl.MIN_COMBO_FOR_BONUS * 2) : int(GameControl.MIN_COMBO_FOR_BONUS);
               if(_comboCount >= _loc6_)
               {
                  createSprite(BonusSprite);
               }
            }
            _comboCount = 0;
         }
         _loc2_ = _gameScreen.spriteHolder.numChildren;
         _loc3_ = 0;
         _loc4_ = true;
         _loc5_ = 0;
         while(_loc5_ < _loc2_)
         {
            if(!((_loc7_ = GameSprite(_gameScreen.spriteHolder.getChildAt(_loc5_))) is Player))
            {
               if(_loc7_ is BonusSprite)
               {
                  _loc3_++;
               }
               else
               {
                  Enemy(_loc7_).calculateSpeed(_player);
               }
            }
            _loc5_++;
         }
         if(_gameMode == GameControl.GAME_MODE_ARCADE)
         {
            _loc8_ = new Date();
            _currentTime = _loc8_.getTime() - _timeStarted.getTime();
            _timeRemaining = Config.TIMER_MODE_TIMEPERGAME * 1000 - _currentTime + _pausedTime;
            if(_timeRemaining <= 0)
            {
               gameOver();
            }
         }
         if(_loc2_ - 1 < GameControl.MAX_ENEMIES && Math.random() < GameControl.ENEMY_CREATION_CHANCE)
         {
            createSprite(Enemy);
         }
      }
      
      private function spritesHittingHandler(param1:CustomEvent) : void
      {
         if(param1.data.target is Player)
         {
            if(param1.data.sprite is Enemy)
            {
               ++_player.enemiesHitThisJump;
               _player.dispatchEvent(new CustomEvent(GameSprite.EVENT_ENEMY_HIT,{"enemy":param1.data.sprite}));
            }
            else if(param1.data.sprite is BonusSprite)
            {
               _player.dispatchEvent(new CustomEvent(GameSprite.EVENT_BONUS_HIT,{"bonus":param1.data.sprite}));
            }
         }
      }
   }
}
