package marvel.centralsmashdown.game.sprites
{
   import flash.display.MovieClip;
   import marvel.centralsmashdown.game.control.GameControl;
   
   public class Enemy extends GameSprite
   {
      
      public static var FOLLOWER_MIN_RADIUS:int = 50;
      
      public static var MAX_SPEED_X:Number = 5;
      
      public static var MAX_SPEED_Y:Number = 2.5;
      
      public static var AIM_CHANGE_CHANCE:Number = 0.02;
      
      public static var SNAKE_AMPLITUDE:Number = 3;
      
      public static var SPEED_AIM_RATE:int = 20;
       
      
      public var distance:Number;
      
      private var _amplitude:Number;
      
      private var _count:int = 0;
      
      public var enemyType:String;
      
      public var char:MovieClip;
      
      public var aimSpeedY:Number;
      
      public var opp:Number;
      
      public var aimSpeedX:Number;
      
      public var adj:Number;
      
      public var angle:Number;
      
      public function Enemy()
      {
         _count = 0;
         super();
         maxYPos = GameControl.ENEMY_MAX_Y;
         maxXPos = GameControl.GAME_WIDTH + GameControl.ENEMY_START_PADDING_X;
         minXPos = -GameControl.ENEMY_START_PADDING_X;
      }
      
      public function chooseType() : void
      {
         var _loc1_:Number = NaN;
         while(enemyType == null || GameControl.lastEnemyType == enemyType)
         {
            _loc1_ = Math.random();
            if(_loc1_ < 1 / 8)
            {
               enemyType = GameControl.ENEMY_TYPE_FOLLOWER;
            }
            else if(_loc1_ < 3 / 8)
            {
               enemyType = GameControl.ENEMY_TYPE_FLOATER;
            }
            else if(_loc1_ < 6 / 8)
            {
               enemyType = GameControl.ENEMY_TYPE_MOVER;
            }
            else
            {
               enemyType = GameControl.ENEMY_TYPE_SNAKE;
            }
         }
         char.gotoAndStop(enemyType);
      }
      
      private function getRandomSpeed() : void
      {
         aimSpeedX = Math.random() * (Enemy.MAX_SPEED_X * 2) - Enemy.MAX_SPEED_X;
         aimSpeedY = Math.random() * (Enemy.MAX_SPEED_Y * 2) - Enemy.MAX_SPEED_Y;
         if(aimSpeedX < 0 && x < -GameControl.ENEMY_START_PADDING_X || aimSpeedX > 0 && x > GameControl.GAME_WIDTH + GameControl.ENEMY_START_PADDING_X)
         {
            aimSpeedX = -aimSpeedX;
         }
      }
      
      public function init(param1:Player) : *
      {
         var _loc2_:Number = NaN;
         speedX = 0;
         speedY = 0;
         accelerationX = 0;
         accelerationY = 0;
         resistanceX = 1;
         resistanceY = 1;
         chooseType();
         _loc2_ = Math.random() * GameControl.ENEMY_CREATION_RANGE + GameControl.ENEMY_START_PADDING_Y;
         if(param1.isGrounded == true || param1.speedY < 0)
         {
            _loc2_ = -_loc2_;
         }
         y = param1.y + _loc2_;
         switch(enemyType)
         {
            case GameControl.ENEMY_TYPE_FOLLOWER:
            case GameControl.ENEMY_TYPE_FLOATER:
               x = Math.random() * GameControl.GAME_WIDTH;
               break;
            case GameControl.ENEMY_TYPE_MOVER:
            case GameControl.ENEMY_TYPE_SNAKE:
               x = Math.random() > 0.5 ? -GameControl.ENEMY_START_PADDING_X : GameControl.GAME_WIDTH + GameControl.ENEMY_START_PADDING_X;
               _amplitude = enemyType == GameControl.ENEMY_TYPE_MOVER ? 0 : Enemy.SNAKE_AMPLITUDE;
               scaleX = enemyType == GameControl.ENEMY_TYPE_MOVER && x == -GameControl.ENEMY_START_PADDING_X ? -1 : 1;
         }
         resetAim(param1);
      }
      
      public function calculateSpeed(param1:GameSprite) : void
      {
         var _loc2_:Object = null;
         _loc2_ = calculateRelativePosition(param1);
         adj = _loc2_.adj;
         opp = _loc2_.opp;
         angle = _loc2_.angle;
         distance = _loc2_.distance;
         speedX += (aimSpeedX - speedX) / Enemy.SPEED_AIM_RATE;
         speedY += (aimSpeedY - speedY) / Enemy.SPEED_AIM_RATE;
         if(speedX > Enemy.MAX_SPEED_X)
         {
            speedX = Enemy.MAX_SPEED_X;
         }
         if(speedY > Enemy.MAX_SPEED_Y)
         {
            speedY = Enemy.MAX_SPEED_Y;
         }
         if(speedX < -Enemy.MAX_SPEED_X)
         {
            speedX = -Enemy.MAX_SPEED_X;
         }
         if(speedY < -Enemy.MAX_SPEED_Y)
         {
            speedY = -Enemy.MAX_SPEED_Y;
         }
         switch(enemyType)
         {
            case GameControl.ENEMY_TYPE_FOLLOWER:
            case GameControl.ENEMY_TYPE_FLOATER:
               if(Math.random() < Enemy.AIM_CHANGE_CHANCE)
               {
                  resetAim(param1);
               }
               break;
            case GameControl.ENEMY_TYPE_MOVER:
            case GameControl.ENEMY_TYPE_SNAKE:
               resetAim(param1);
         }
      }
      
      override public function setToMaxY() : void
      {
         switch(enemyType)
         {
            case GameControl.ENEMY_TYPE_FLOATER:
               if(aimSpeedY > 0)
               {
                  aimSpeedY = -aimSpeedY;
               }
               break;
            case GameControl.ENEMY_TYPE_MOVER:
         }
         super.setToMaxY();
      }
      
      public function resetAim(param1:GameSprite) : *
      {
         var _loc2_:* = undefined;
         var _loc3_:Number = NaN;
         var _loc4_:Number = NaN;
         switch(enemyType)
         {
            case GameControl.ENEMY_TYPE_FOLLOWER:
               if(Player(param1).isGrounded != true)
               {
                  _loc2_ = Math.random() * (Math.PI * 2);
                  _loc3_ = param1.x + Math.sin(_loc2_) * Enemy.FOLLOWER_MIN_RADIUS;
                  _loc4_ = param1.y + Math.cos(_loc2_) * Enemy.FOLLOWER_MIN_RADIUS;
                  aimSpeedX = (_loc3_ - x) / Enemy.SPEED_AIM_RATE;
                  aimSpeedY = (_loc4_ - y) / Enemy.SPEED_AIM_RATE;
               }
               else
               {
                  getRandomSpeed();
               }
               break;
            case GameControl.ENEMY_TYPE_FLOATER:
               getRandomSpeed();
               break;
            case GameControl.ENEMY_TYPE_MOVER:
            case GameControl.ENEMY_TYPE_SNAKE:
               if(isNaN(aimSpeedX))
               {
                  aimSpeedX = this.x < 0 ? Enemy.MAX_SPEED_X : -Enemy.MAX_SPEED_X;
               }
               ++_count;
               aimSpeedY = _amplitude * Math.sin(Math.PI / 180 * _count);
         }
      }
   }
}
