package marvel.centralsmashdown.game.sprites
{
   import com.markdavies.events.CustomEvent;
   import flash.display.*;
   import flash.errors.*;
   import flash.events.*;
   import marvel.centralsmashdown.game.control.*;
   
   public class GameSprite extends MovieClip
   {
      
      public static var EVENT_GROUND_HIT:String = "groundHit";
      
      public static var EVENT_ENEMY_DIE:String = "enemyDie";
      
      public static var EVENT_PLAYER_JUMP:String = "playerJump";
      
      public static var STATUS_ALIVE:String = "alive";
      
      public static var EVENT_OUTSIDE_BORDERS:String = "outsideBorders";
      
      public static var STATUS_DEAD:String = "dead";
      
      public static var EVENT_IS_HITTING_SPRITE:String = "spritesHitting";
      
      public static var EVENT_ENEMY_HIT:String = "enemyHit";
      
      public static var EVENT_BONUS_HIT:String = "bonusHit";
      
      public static var STATUS_CREATING:String = "creating";
       
      
      public var accelerationX:Number;
      
      public var minXPos:Number;
      
      public var resistanceX:Number;
      
      public var resistanceY:Number;
      
      public var maxYPos:Number;
      
      public var maxXPos:Number;
      
      public var currentStatus:String;
      
      public var accelerationY:Number;
      
      public var speedX:Number;
      
      public var speedY:Number;
      
      public function GameSprite()
      {
         super();
         currentStatus = GameSprite.STATUS_ALIVE;
         this.addEventListener(GameSprite.EVENT_ENEMY_DIE,dieHandler);
      }
      
      public function checkGroundHit() : void
      {
         if(this.y > maxYPos)
         {
            dispatchEvent(new CustomEvent(GameSprite.EVENT_GROUND_HIT,{"target":this}));
         }
      }
      
      public function checkOtherSprites() : Boolean
      {
         var _loc1_:Object = null;
         var _loc2_:Boolean = false;
         var _loc3_:Number = NaN;
         var _loc4_:GameSprite = null;
         _loc2_ = true;
         if(this.parent == null)
         {
            return false;
         }
         _loc3_ = 0;
         while(_loc3_ < this.parent.numChildren)
         {
            _loc4_ = GameSprite(this.parent.getChildAt(_loc3_));
            _loc1_ = calculateRelativePosition(_loc4_);
            if(_loc4_ != this && _loc1_.distance < GameControl.HIT_DISTANCE && _loc4_.currentStatus == GameSprite.STATUS_ALIVE)
            {
               _loc2_ = true;
               dispatchEvent(new CustomEvent(GameSprite.EVENT_IS_HITTING_SPRITE,{
                  "target":this,
                  "sprite":_loc4_,
                  "pos":_loc1_
               }));
            }
            else if(this is Player && _loc1_.distance > GameControl.ENEMY_REMOVE_RANGE)
            {
               if(_loc4_ is Enemy || _loc4_ is BonusSprite)
               {
                  _loc4_.die();
               }
            }
            _loc3_++;
         }
         return _loc2_;
      }
      
      public function checkOutsideBorders() : void
      {
         if(this.x < minXPos || this.x > maxXPos)
         {
            dispatchEvent(new CustomEvent(GameSprite.EVENT_OUTSIDE_BORDERS,{
               "target":this,
               "direction":this.x > maxXPos
            }));
         }
      }
      
      public function setToMaxY() : void
      {
         this.y = maxYPos;
      }
      
      public function die() : void
      {
         try
         {
            parent.removeChild(this);
         }
         catch(e:TypeError)
         {
            trace("cant find " + this.name);
         }
      }
      
      private function dieHandler(param1:Event) : *
      {
         die();
      }
      
      public function calculateRelativePosition(param1:GameSprite) : Object
      {
         var _loc2_:Number = NaN;
         var _loc3_:Number = NaN;
         var _loc4_:Number = NaN;
         var _loc5_:Number = NaN;
         _loc2_ = param1.x - x;
         _loc3_ = param1.y - y;
         _loc4_ = Math.atan2(_loc2_,_loc3_);
         _loc5_ = Math.sqrt(Math.pow(_loc2_,2) + Math.pow(_loc3_,2));
         return {
            "adj":_loc2_,
            "opp":_loc3_,
            "angle":_loc4_,
            "distance":_loc5_
         };
      }
      
      public function move() : void
      {
         speedX += accelerationX;
         speedY += accelerationY;
         accelerationX *= resistanceX;
         accelerationY *= resistanceY;
         this.x += speedX;
         this.y += speedY;
         checkGroundHit();
         checkOutsideBorders();
      }
   }
}
