package marvel.centralsmashdown.game.sprites
{
   import com.gs.TweenLite;
   import com.markdavies.events.CustomEvent;
   import flash.display.*;
   import flash.events.*;
   import flash.geom.*;
   import marvel.centralsmashdown.*;
   import marvel.centralsmashdown.game.control.GameControl;
   
   public class Player extends GameSprite
   {
      
      public static var STATUS_IDLE:String = "idle";
      
      public static var MIN_JUMP_SPEED:Number = -10;
      
      public static var DOUBLE_UP_TIME:Number = 10000;
      
      public static var MIN_POWER_FOR_JUMP:Number = 0.15;
      
      public static var RECHARGE_POWER_RATE:Number = 0.0125;
      
      public static var STATUS_FLY:String = "fly";
      
      public static var TIME_POWERBAR_FADEUP:Number = 0.5;
      
      public static var STATUS_LAND:String = "land";
      
      public static var STATUS_JUMP:String = "jump";
      
      public static var STATUS_DROP:String = "drop";
      
      public static var MAX_JUMP_SPEED:Number = -25;
      
      public static var STATUS_POWERUP:String = "powerup";
      
      public static var MOUSE_FOLLOW_RATE:Number = 0.035;
      
      public static var DEFAULT_ACCELERATION_Y:Number = 0.4;
      
      public static var ANIM_DROP_SPEEDY:Number = -15;
       
      
      private var _power:Number;
      
      private var _status:String;
      
      private var _isDoubleUp:Boolean;
      
      public var doubleUpBar:MovieClip;
      
      private var _gameControl:GameControl;
      
      private var _mouseIsDown:Boolean;
      
      private var _doubleUpStart:Date;
      
      public var inner:MovieClip;
      
      private var _mouseY:Number;
      
      public var enemiesHitThisJump:Number;
      
      public var isGrounded:Boolean;
      
      private var _main:Main;
      
      public var powerBar:MovieClip;
      
      private var _mouseX:Number;
      
      public var dropHeight:Number;
      
      public function Player()
      {
         super();
         maxYPos = GameControl.GROUND_Y;
         maxXPos = GameControl.GAME_WIDTH - 50;
         minXPos = 50;
         _gameControl = GameControl.getInstance();
         _main = Main.getInstance();
      }
      
      public function set isDoubleUp(param1:Boolean) : void
      {
         _isDoubleUp = param1;
         if(param1 == true)
         {
            _doubleUpStart = new Date();
            TweenLite.to(doubleUpBar,Player.TIME_POWERBAR_FADEUP,{"alpha":1});
         }
         else
         {
            TweenLite.to(doubleUpBar,Player.TIME_POWERBAR_FADEUP,{"alpha":0});
         }
      }
      
      public function set power(param1:Number) : void
      {
         _power = param1;
      }
      
      public function jump(param1:Number) : void
      {
         var _loc2_:int = 0;
         if(isGrounded == true)
         {
            _loc2_ = Math.round(Math.random() * 4) + 1;
            _main.sc.play("s_jump" + _loc2_.toString());
         }
         dropHeight = 0;
         isGrounded = false;
         speedY = Player.MIN_JUMP_SPEED + (Player.MAX_JUMP_SPEED - Player.MIN_JUMP_SPEED) * param1;
         accelerationY = Player.DEFAULT_ACCELERATION_Y;
         status = Player.STATUS_JUMP;
         dispatchEvent(new CustomEvent(GameSprite.EVENT_PLAYER_JUMP));
      }
      
      public function mouseReleaseHandler(param1:MouseEvent) : void
      {
         if(_mouseIsDown == true && isGrounded == true && _power > Player.MIN_POWER_FOR_JUMP)
         {
            jump(_power);
         }
         else if(_gameControl.paused != true && speedY == 0)
         {
            status = Player.STATUS_IDLE;
            _power = 0;
         }
         TweenLite.to(powerBar,Player.TIME_POWERBAR_FADEUP,{"alpha":0});
         _mouseIsDown = false;
      }
      
      public function reset() : void
      {
         x = GameControl.GAME_WIDTH / 2;
         setToMaxY();
         _power = 0;
         powerBar.alpha = 0;
         doubleUpBar.alpha = 0;
         status = Player.STATUS_IDLE;
      }
      
      public function checkDoubleUp() : void
      {
         var _loc1_:Date = null;
         var _loc2_:int = 0;
         if(_isDoubleUp == true)
         {
            _loc1_ = new Date();
            _loc2_ = _loc1_.getTime() - _doubleUpStart.getTime();
            doubleUpBar.bar.scaleX = 1 - _loc2_ / Player.DOUBLE_UP_TIME;
            if(_loc2_ > Player.DOUBLE_UP_TIME)
            {
               isDoubleUp = false;
            }
         }
      }
      
      override public function setToMaxY() : void
      {
         super.setToMaxY();
         enemiesHitThisJump = 0;
         accelerationX = 0;
         accelerationY = 0;
         resistanceX = 0.98;
         resistanceY = 1;
         speedX = 0;
         speedY = 0;
         status = Player.STATUS_LAND;
         isGrounded = true;
         if(_mouseIsDown == true)
         {
            TweenLite.to(powerBar,Player.TIME_POWERBAR_FADEUP,{"alpha":1});
         }
      }
      
      public function set status(param1:String) : void
      {
         if(param1 != _status)
         {
            _status = param1;
            inner.gotoAndPlay(param1);
         }
      }
      
      public function get power() : Number
      {
         return _power;
      }
      
      public function get isDoubleUp() : Boolean
      {
         return _isDoubleUp;
      }
      
      public function mousePressHandler(param1:MouseEvent) : void
      {
         _mouseIsDown = true;
         if(isGrounded == true && _gameControl.paused == false)
         {
            TweenLite.to(powerBar,Player.TIME_POWERBAR_FADEUP,{"alpha":1});
            status = Player.STATUS_POWERUP;
         }
      }
      
      override public function move() : void
      {
         if(isGrounded == false)
         {
            speedX = this.mouseX * Player.MOUSE_FOLLOW_RATE;
            if(speedY > Player.ANIM_DROP_SPEEDY)
            {
               status = Player.STATUS_DROP;
            }
            inner.scaleX = speedX > 0 ? 1 : -1;
         }
         if(y < dropHeight)
         {
            dropHeight = y;
         }
         super.move();
      }
      
      public function checkPower() : void
      {
         if(_mouseIsDown == true && isGrounded == true)
         {
            _power += Player.RECHARGE_POWER_RATE;
            powerBar.bar.scaleX = _power;
            if(_power >= 1)
            {
               _power = 1;
               jump(_power);
               TweenLite.to(powerBar,Player.TIME_POWERBAR_FADEUP,{"alpha":0});
            }
         }
      }
   }
}
